@@ -56,13 +56,19 @@ C<encode_ctcp> and C<decode_ctcp> provided by L<AnyEvent::IRC::Util>.
 
 This constructor doesn't take any arguments.
 
+B<NOTE:> You are free to use the hash member C<heap> (which contains a hash) to
+store any associated data with this object. For example retry timers or
+anything else.
+
+You can also access that member via the C<heap> method.
+
 =cut
 
 sub new {
   my $this = shift;
   my $class = ref($this) || $this;
 
-  my $self = $class->SUPER::new (@_);
+  my $self = $class->SUPER::new (@_, heap => { });
 
   bless $self, $class;
 
@@ -76,7 +82,7 @@ sub new {
   return $self;
 }
 
-=item $con->connect ($host, $port)
+=item $con->connect ($host, $port [, $prepcb_or_timeout])
 
 Tries to open a socket to the host C<$host> and the port C<$port>.
 If an error occurred it will die (use eval to catch the exception).
@@ -84,50 +90,56 @@ If an error occurred it will die (use eval to catch the exception).
 If you want to connect via TLS/SSL you have to call the C<enable_ssl>
 method before to enable it.
 
+C<$prepcb_or_timeout> can either be a callback with the semantics of a prepare
+callback for the function C<tcp_connect> in L<AnyEvent::Socket> or a simple
+number which stands for a timeout.
+
 =cut
 
 sub connect {
-   my ($self, $host, $port) = @_;
+   my ($self, $host, $port, $prep) = @_;
 
-   $self->{socket}
-      and return;
+   if ($self->{socket}) {
+      $self->disconnect ("reconnect requested.");
+   }
 
-   tcp_connect $host, $port, sub {
-      my ($fh) = @_;
+   $self->{con_guard} =
+      tcp_connect $host, $port, sub {
+         my ($fh) = @_;
 
-      delete $self->{socket};
+         delete $self->{socket};
 
-      unless ($fh) {
-         $self->event (connect => $!);
-         return;
-      }
+         unless ($fh) {
+            $self->event (connect => $!);
+            return;
+         }
 
-      $self->{host} = $host;
-      $self->{port} = $port;
+         $self->{host} = $host;
+         $self->{port} = $port;
 
-      $self->{socket} =
-         AnyEvent::Handle->new (
-            fh => $fh,
-            ($self->{enable_ssl} ? (tls => 'connect') : ()),
-            on_eof => sub {
-               $self->disconnect ("EOF from server $host:$port");
-            },
-            on_error => sub {
-               $self->disconnect ("error in connection to server $host:$port: $!");
-            },
-            on_read => sub {
-               my ($hdl) = @_;
-               $hdl->push_read (line => sub {
-                  $self->_feed_irc_data ($_[1]);
-               });
-            },
-            on_drain => sub {
-               $self->event ('buffer_empty');
-            }
-         );
+         $self->{socket} =
+            AnyEvent::Handle->new (
+               fh => $fh,
+               ($self->{enable_ssl} ? (tls => 'connect') : ()),
+               on_eof => sub {
+                  $self->disconnect ("EOF from server $host:$port");
+               },
+               on_error => sub {
+                  $self->disconnect ("error in connection to server $host:$port: $!");
+               },
+               on_read => sub {
+                  my ($hdl) = @_;
+                  $hdl->push_read (line => sub {
+                     $self->_feed_irc_data ($_[1]);
+                  });
+               },
+               on_drain => sub {
+                  $self->event ('buffer_empty');
+               }
+            );
 
-      $self->event ('connect');
-   };
+         $self->event ('connect');
+      }, (defined $prep ? (ref $prep ? $prep : sub { $prep }) : ());
 }
 
 =item $con->enable_ssl ()
@@ -150,7 +162,8 @@ the sockets and send a 'disconnect' event with C<$reason> as argument.
 
 sub disconnect {
    my ($self, $reason) = @_;
-   return unless $self->{socket};
+
+   delete $self->{con_guard};
    delete $self->{socket};
    $self->event (disconnect => $reason);
 }
@@ -169,8 +182,8 @@ sub is_connected {
 
 =item $con->heap ()
 
-Returns a hash reference that is local to this connection object
-that lets you store any information you want.
+Returns the hash reference stored in the C<heap> member, that is local to this
+connection object that lets you store any information you want.
 
 =cut
 
